/**
* A simple benchmark for remote invocations.
* It takes the system clock right before and after 
* method invocations.
* This class is changed by H. Timucin Ozdemir.
* Original version can be found with JacORBv0.8d. 
*/

import java.net.*;
import java.io.*;

import BenchmarkPackage.*;

public class 
Benchmarker 
{
    /**
    */
    void
    displayIntArray(String pre, int val[], String post)
    {
       System.out.print(pre);
       for(int i=0;i<val.length;i++) System.out.print(val[i]+" ");
       System.out.println(post);

    }// end of displayIntArray()

    /**
    */
    Stat
    doIntArrayFor(Benchmark benchmarkInstance, int loop, int size)
    {
        Stat ret = new Stat();
        int nb = loop;
        int myInt[] = new int[size];
        int rmyInt[] ;
        long stopTime, startTime, diff;
        long total=0;

        for( int si = 0; si < size; si++) { myInt[si]=si; }

        displayIntArray(" I will send : ",myInt,"");
        displayIntArray(" I received : ",benchmarkInstance.echoLongArray(myInt),"");

        ret.min   = Float.MAX_VALUE;
        ret.max   = Float.MIN_VALUE;

        ret.start = System.currentTimeMillis();
        while (nb-- > 0) 
        {

           //startTime = System.currentTimeMillis();

           rmyInt = benchmarkInstance.echoLongArray(myInt);

           /*stopTime = System.currentTimeMillis();

           diff = stopTime-startTime;

           System.out.println("diff ["+ nb +"] = " + diff); 

           if( diff<ret.min ) ret.min = diff;
           if( diff>ret.max ) ret.max = diff;

           total += diff;
           */

        }// end of while

        ret.stop = System.currentTimeMillis();
        total = ret.stop -  ret.start ;
        
        //ret.average = ((ret.stop-ret.start) / (float)loop);
        ret.average = (total / (float)loop);
        ret.timePerItem = ( ret.average / (float)size);

        return ret;

    }// end of doIntArrayFor()

    /**
    */
    public void
    doIntArray(Benchmark benchmarkInstance) throws java.io.IOException
    {
        DataInput d = new DataInputStream(System.in);
        System.out.print("     Number of loops : ");
        int loop = new Integer(d.readLine()).intValue(); 
        int nb = loop;

        System.out.print("     Size of array : ");
        int size = new Integer(d.readLine()).intValue(); 

        Stat ret = doIntArrayFor(benchmarkInstance,loop,size);

        System.out.println(">>> Elapsed time = " + (ret.stop - ret.start)/1000 
		       + " secs      Average time = " + ret.average + " msecs"
		       + " Minimum time = " + ret.min + " msecs"
		       + " Maximum time = " + ret.max + " msecs"
		       + " Average Time per item = " + ret.timePerItem + " msecs"
               );

    }// end of doIntArray()

    /**
    */
    void
    displayStructArray(String pre, long_struct val[], String post)
    {
       System.out.print(pre);
       for(int i=0;i<val.length;i++) System.out.print(val[i].number+" ");
       System.out.println(post);

    }// end of displayIntArray()

    /**
    */
    Stat
    doStructArrayFor(Benchmark benchmarkInstance, int loop, int size)
    {
        Stat ret = new Stat();
        int nb = loop;
        long stopTime, startTime, diff;
        long total=0;

        long_struct myInt[] = new long_struct[size];

        for( int si = 0; si < size; si++)
        { myInt[si] = new long_struct(si); }

        displayStructArray(" I will send ",myInt," ");
        displayStructArray(" I received ",benchmarkInstance.echoLong_structArray(myInt)," ");

        ret.min = Float.MAX_VALUE;
        ret.max = Float.MIN_VALUE;
        ret.start = System.currentTimeMillis();

        while (nb-- > 0) 
        {
           //startTime = System.currentTimeMillis();
           benchmarkInstance.echoLong_structArray(myInt);
     
           /*
           stopTime = System.currentTimeMillis();

           diff = stopTime-startTime;

           System.out.println("diff ["+ nb +"] = " + diff); 

           if( diff<ret.min ) ret.min = diff;
           if( diff>ret.max ) ret.max = diff;

           total += diff;
           */ 

        }// end of while
        ret.stop = System.currentTimeMillis();
        total = ret.stop - ret.start;
        //ret.average = ((ret.stop-ret.start) / (float)loop);
        ret.average = ((total) / (float)loop);
        ret.timePerItem = ( ret.average / (float)size);

        return ret;

    }// end of doStructArrayFor()

    /**
    */
    public void
    doStructArray(Benchmark benchmarkInstance) throws java.io.IOException
    {
        DataInput d = new DataInputStream(System.in);
        System.out.print("     Number of loops : ");
        int loop = new Integer(d.readLine()).intValue(); 
        int nb = loop;
        System.out.print("     Size of array : ");
        int size = new Integer(d.readLine()).intValue(); 

        Stat ret = doStructArrayFor(benchmarkInstance,loop,size);

        System.out.println(">>> Elapsed time = " + (ret.stop - ret.start)/1000 
		       + " secs      Average time = " + ret.average + " msecs"
		       + " Minimum time = " + ret.min + " msecs"
		       + " Maximum time = " + ret.max + " msecs"
		       + " Average Time per item = " + ret.timePerItem + " msecs"
               );

    }// end of doStructArray()

    /**
    */
    Stat
    doPingFor(Benchmark benchmarkInstance, int loop)
    {

      Stat ret = new Stat();
      int nb = loop;
  
      long min = 200000;
      long startTime = 0;
      long stopTime = 0;
      long total = 0;
      long diff;
      ret.min = Float.MAX_VALUE;
      ret.max = Float.MIN_VALUE;

      System.out.println("     ping for "+loop+" times!! ");

      benchmarkInstance.ping();

      /*
      while (nb-- > 0)
      {
         startTime = System.currentTimeMillis();
         benchmarkInstance.ping();
         stopTime = System.currentTimeMillis();

         diff = stopTime-startTime;
         System.out.println("diff ["+ nb +"] = " + diff); 
         
         if( diff<ret.min ) ret.min = diff;
         if( diff>ret.max ) ret.max = diff;

         total += diff;

       }// end of while
       */

      startTime = System.currentTimeMillis();
      while (nb-- > 0) benchmarkInstance.ping();
      stopTime = System.currentTimeMillis();
      total = stopTime-startTime;

       ret.average = (total/(float)loop);
 
       return ret;

    }// end of doPingFor()

    /**
    */
    public void
    doPing(Benchmark benchmarkInstance) throws java.io.IOException
    {
        DataInput d = new DataInputStream(System.in);
        System.out.print("     Number of loops : ");
        int loop = new Integer(d.readLine()).intValue();

        Stat result = doPingFor(benchmarkInstance,loop);

        System.out.println(">>> Elapsed Minimum time = " + result.min 
               + " msecs      Average time = " + result.average + " msecs"
		       + " Maximum time = " + result.max + " msecs");
		    
    }// end of doPing()

    /**
    */
    String
    displayMenu() throws java.io.IOException
    {
       String line = null;
    
       while(line==null)
       {
          System.out.print("     Ping                       [1]\n"
                          +"     Transfer array of int      [2]\n"
                          +"     Transfer array of struct   [3]\n"
                          +"                                   \n"
                          +"     Auto mode     - Long! -    [auto]\n"
                          +"     EXIT                       [0]\n"
                          +"     --------------------------------\n"
                          +"         Your choice :          ");
          DataInput d = new DataInputStream(System.in);
          line = d.readLine();

        }// end of while

        return line;

    }// enmd of displayMenu()

    /**
    */
    public void
    doAuto(Benchmark benchmarkInstance) throws java.io.IOException
    {

        DataInput d = new DataInputStream(System.in);
        System.out.println("#### Entering auto-mode ####");
        System.out.print("     Number of loops : ");
        int loop = new Integer(d.readLine()).intValue();

        System.out.print("     Number of iterations  : ");
        int iter = new Integer(d.readLine()).intValue();

        System.out.print("     Step size : ");
        int step = new Integer(d.readLine()).intValue();

        int size = 1;
        System.out.println("\n Results are average times in msecs for "
                           + loop+ " round trips\n");
        System.out.println(" Size of array   Ping   Array of int Array of struct");
        System.out.println(" ============= ======== ============ ===============");

        Stat pingRes[] = new Stat[iter];
        Stat intArrRes[] = new Stat[iter];
        Stat structArrRes[] = new Stat[iter];

        for (int i=0;i<iter;i++) 
        {
            System.out.print("\t"+size);

            pingRes[i] = doPingFor(benchmarkInstance,loop);
            System.out.print("\t"+pingRes[i].average);

            intArrRes[i] = doIntArrayFor(benchmarkInstance,loop,size);
            System.out.print("\t"+intArrRes[i].average);

            structArrRes[i] = doStructArrayFor(benchmarkInstance,loop,size);
            System.out.println("\t\t"+structArrRes[i].average);

            size = (i+1)*step;

        }// end of for

        System.out.println("\n#### Exiting auto-mode ####\n");

        doReport(pingRes,intArrRes,structArrRes,System.out,iter,step,loop);

        doReportForMinimumTimes(pingRes,intArrRes,structArrRes,
                                System.out,iter,step,loop,false);

        doReportForAverageTimePerItemTransfer(pingRes,intArrRes,structArrRes,
                                System.out,iter,step,loop,false);

        File file = new File("benchResults.html");
        PrintStream fout = new PrintStream(new FileOutputStream(file));
        doReportHTML(pingRes,intArrRes,structArrRes,fout,iter,step,loop);

        doReportForMinimumTimes(pingRes,intArrRes,structArrRes,
                                System.out,iter,step,loop,true);

        doReportForAverageTimePerItemTransfer(pingRes,intArrRes,structArrRes,
                                System.out,iter,step,loop,true);


    }// end of doAuto()

    /**
    */
    void
    doReportForMinimumTimes(Stat pingRes[], Stat intArrRes[], 
                            Stat structArrRes[], PrintStream pstr, int iter, 
                            int step, int loop, boolean isHTML)
                            throws java.io.IOException
    {
       float arr1[] = new float[iter];
       float arr2[] = new float[iter];
       float arr3[] = new float[iter];

       for(int i=0;i<iter;i++)
       {
          arr1[i] = pingRes[i].min;
          arr2[i] = intArrRes[i].min;
          arr3[i] = structArrRes[i].min;

       }// end of for

       String header = new String("\n Results are minimum trip times in msecs for ");

       if(isHTML)
       {
          File file = new File("benchMinResults.html");
          PrintStream fout = new PrintStream(new FileOutputStream(file));
          doReportHTML(arr1,arr2,arr3,header,fout,iter,step,loop);

       } else doReport(arr1,arr2,arr3,header,System.out,iter,step,loop);

    }// end of doReportForMinimumTimes()

    /**
    */
    void
    doReportForMaximumTimes(Stat pingRes[], Stat intArrRes[], 
                            Stat structArrRes[], PrintStream pstr, int iter, 
                            int step, int loop, boolean isHTML)
                            throws java.io.IOException
    {
       float arr1[] = new float[iter];
       float arr2[] = new float[iter];
       float arr3[] = new float[iter];

       for(int i=0;i<iter;i++)
       {
          arr1[i] = pingRes[i].max;
          arr2[i] = intArrRes[i].max;
          arr3[i] = structArrRes[i].max;

       }// end of for

       String header = new String("\n Results are maximum trip times in msecs for ");

       if(isHTML)
       {
          File file = new File("benchMaxResults.html");
          PrintStream fout = new PrintStream(new FileOutputStream(file));
          doReportHTML(arr1,arr2,arr3,header,fout,iter,step,loop);

       } else doReport(arr1,arr2,arr3,header,System.out,iter,step,loop);

    }// end of doReportForMaximumTimes()

    /**
    */
    void
    doReportForAverageTimePerItemTransfer(Stat pingRes[], Stat intArrRes[], 
                            Stat structArrRes[], PrintStream pstr, int iter, 
                            int step, int loop, boolean isHTML)
                            throws java.io.IOException
    {
       float arr1[] = new float[iter];
       float arr2[] = new float[iter];
       float arr3[] = new float[iter];

       for(int i=0;i<iter;i++)
       {
          arr1[i] = pingRes[i].timePerItem;
          arr2[i] = intArrRes[i].timePerItem;
          arr3[i] = structArrRes[i].timePerItem;

       }// end of for

       String header = new String("\n Results are average time per item transfer in msecs for ");

       if(isHTML)
       {
          File file = new File("benchAveTmPerItemResults.html");
          PrintStream fout = new PrintStream(new FileOutputStream(file));
          doReportHTML(arr1,arr2,arr3,header,fout,iter,step,loop);

       } else doReport(arr1,arr2,arr3,header,System.out,iter,step,loop);

    }// end of doReportForAverageTimePerItemTransfer()

    /**
    */
    void
    doReport(Stat pingRes[], Stat intArrRes[], Stat structArrRes[],
             PrintStream pstr, int iter, int step, int loop)
    {
        pstr.println("\n Results are average times in msecs for "
                      + loop+ " round trips\n");
        pstr.println(" Size of array   Ping   Array of int Array of struct");
        pstr.println(" ============= ======== ============ ===============");

        int size = 1;

        for (int i=0;i<iter;i++) 
        {
           pstr.print("\t"+size);

           pstr.print("\t"+pingRes[i].average);

           pstr.print("\t"+intArrRes[i].average);

           pstr.println("\t\t"+structArrRes[i].average);

           //size = size*step;
           size = (i+1)*step;

        }// end of for

        pstr.println("\n#### Exiting auto-mode ####\n");

    }// end of doReport()

    /**
    */
    void
    doReport(float arr1[], float arr2[], float arr3[], String header,
             PrintStream pstr, int iter, int step, int loop)
    {
        //pstr.println("\n Results are average times in msecs for "
        pstr.println(header+loop+ " round trips\n");

        pstr.println(" Size of array   Ping   Array of int Array of struct");
        pstr.println(" ============= ======== ============ ===============");

        int size = 1;

        //for (int i=0;i<iter;i++,size = size*step) 
        for (int i=0;i<iter;i++,size = i*step) 
           pstr.println("\t"+size+"\t"+arr1[i]+"\t"+arr2[i]+"\t\t"+arr3[i]);

        pstr.println("\nEnd of Report\n");

    }// end of doReport()

    /**
    */
    void
    doReportHTML(Stat pingRes[], Stat intArrRes[], Stat structArrRes[],
             PrintStream pstr, int iter, int step, int loop)
    {
        pstr.println("\n <p> ");
        pstr.println("\n <center> ");
        pstr.println("\n Results are average times in msecs for "
                      + loop+ " round trips\n");

        pstr.println("\n <p> ");
        pstr.println("\n <table border=3> ");

        pstr.println("\n <tr align=CENTER> ");

        pstr.print("<td> ");
        pstr.print("Size of array");
        pstr.println("</td> ");

        pstr.print("<td> ");
        pstr.print("Ping");
        pstr.println("</td> ");

        pstr.print("<td> ");
        pstr.print("Array of int");
        pstr.println("</td> ");

        pstr.print("<td> ");
        pstr.print("Array of struct");
        pstr.println("</td> ");

        pstr.println("</tr> ");

        int size = 1;

        for (int i=0;i<iter;i++) 
        {
           pstr.println("\n <tr align=CENTER> ");

           pstr.print("<td> ");
           pstr.print(size);
           pstr.println("</td> ");

           pstr.print("<td> ");
           pstr.print(pingRes[i].average);
           pstr.println("</td> ");

           pstr.print("<td> ");
           pstr.print(intArrRes[i].average);
           pstr.println("</td> ");

           pstr.print("<td> ");
           pstr.println(structArrRes[i].average);
           pstr.println("</td> ");

           //size = size*step;
           size = (i+1)*step;
           pstr.println("</tr> ");

        }// end of for

        pstr.println("\n </table> ");
        pstr.println("\n </center> ");

    }// end of doReport()

    /**
    */
    void
    doReportHTML(float arr1[], float arr2[], float arr3[], String header, 
                 PrintStream pstr, int iter, int step, int loop)
    {
        pstr.println("\n <p> ");
        pstr.println("\n <center> ");
        //pstr.println("\n Results are average times in msecs for "
        pstr.println(header + loop+ " round trips\n");

        pstr.println("\n <p> ");
        pstr.println("\n <table border=3> ");

        pstr.println("\n <tr align=CENTER> ");

        pstr.print("<td> ");
        pstr.print("Size of array");
        pstr.println("</td> ");

        pstr.print("<td> ");
        pstr.print("Ping");
        pstr.println("</td> ");

        pstr.print("<td> ");
        pstr.print("Array of int");
        pstr.println("</td> ");

        pstr.print("<td> ");
        pstr.print("Array of struct");
        pstr.println("</td> ");

        pstr.println("</tr> ");

        int size = 1;

        //for (int i=0;i<iter;i++,size = size*step) 
        for (int i=0;i<iter;i++,size = i*step) 
        {
           pstr.println("\n <tr align=CENTER> ");

           pstr.println("<td> "+size+"</td> ");
           pstr.println("<td> "+arr1[i]+"</td> ");
           pstr.println("<td> "+arr2[i]+"</td> ");
           pstr.println("<td> "+arr3[i]+"</td> ");

           pstr.println("</tr> ");

        }// end of for

        pstr.println("\n </table> ");
        pstr.println("\n </center> ");

    }// end of doReport()

    /**
    */
    public void
    doBenchmarking(Benchmark benchmarkInstance) throws java.io.IOException
    {
       String line=null;
       do
       {
           line = displayMenu();
 
           if(line!=null)
           {
              if ( line.equals("1") ) doPing(benchmarkInstance); 
              else if ( line.equals("2") ) doIntArray(benchmarkInstance); 
              else if ( line.equals("3") ) doStructArray(benchmarkInstance); 
              else if ( line.equals("auto") ) doAuto(benchmarkInstance); 
              else { System.out.println("\nExiting ..."); line=null; }
     
           }// end of if

       } while(line!=null && !line.equals("0"));

    }// end of doBenchmarking()

    /**
    public static void 
    main(String args[]) 
    {
        try 
        {
           if(args.length > 0 && args[args.length-1].startsWith("IOR:") )
           {
 
              // Create and initialize the ORB
              ORB orb = ORB.init(args, null);

              System.out.println(" IOR --> "+args[0]);
              org.omg.CORBA.Object objRef = orb.string_to_object(args[0]);
      
              Benchmark benchRef = BenchmarkHelper.narrow(objRef);
              benchClient client = new benchClient();

              client.doBenchmarking(benchRef);
           }
           else 
            {
               System.err.println("usage: java benchClient <IOR>");
               System.exit(1);
            }

        } catch (Exception e) 
          {
             System.out.println("### Exception !!! ### \n");
             e.printStackTrace();
          }

    }// end of main()
    */

}// end of Class benchClient 
